------------------------------------------------------------------------------
--ScanDoubler pro MZ800-----------------------------------DL2010--------------
------------------------------------------------------------------------------
--Zdrojovy kod je mi uplne volny :-)) ----------------------------------------
--Veskere sireni a pozmenovani je povoleno -----------------------------------
------------------------------------------------------------------------------
library IEEE;
use IEEE.std_logic_1164.ALL;
use IEEE.std_logic_arith.ALL;
use IEEE.std_logic_unsigned.ALL;
entity CGAVGA is port (

--CGA-------------------------------------------------------------------------
CGA_CLK	: in std_logic;						--hodiny 17,734475 MHz
CGA_RGBI: in std_logic_vector(3 downto 0);			--vstup od MZ barvy (I=3,B=2,G=1,R=0)
CGA_HS	: in std_logic;						--vstup od MZ horizontalni synchronizace
CGA_VS	: in std_logic;						--vstup od MZ vertikalni synchronizace
--RAM-------------------------------------------------------------------------
RAM_AD	: out std_logic_vector(17 downto 0);			--adresa k RAM
RAM_DT	: inout std_logic_vector(15 downto 0);			--datova sbernice RAM
RAM_WR	: out std_logic;					--povoleni zapisu do RAM
--VGA-------------------------------------------------------------------------
VGA_CLK	: in std_logic;						--hodiny 40MHz
VGA_R	: out std_logic;					--vga vystup cervena slozka
VGA_G	: out std_logic;					--vga vystup zelena slozka
VGA_B	: out std_logic;					--vga vystup modra slozka
VGA_I	: out std_logic;					--vga vystup intenzita
VGA_HS	: out std_logic;					--vga vystup horizontalni synchronizace
VGA_VS	: out std_logic);					--vga vystup vertikalni synchronizace

end CGAVGA;
architecture rtl of CGAVGA is

--ridici signaly casovani CGA-------------------------------------------------
signal cCGAHOR	: std_logic_vector(9 downto 0);			--horizontalni citac CGA
signal cCGAVER	: std_logic_vector(8 downto 0);			--vertikalni citac CGA
signal rCGAHS	: std_logic;					--pamet stavu horizontalni synchronizace
signal rCGAVS	: std_logic;					--pamet stavu vertikalni synchronizace
signal r3RGBI	: std_logic_vector(11 downto 0);		--pomocna pamet barvy 3 pixelu
signal r4RGBI	: std_logic_vector(15 downto 0);		--pamet barvy 4 pixelu
signal rCGASEL	: std_logic;					--vyber stranky pameti pro zapis

--signaly pro synchronizaci zapisu--------------------------------------------
signal rCGAADR	: std_logic_vector(7 downto 0);			--register adresy pro pamet ram
signal iRAM_WR	: std_logic;					--interni signal /WR pro pamet RAM
signal rWRCOM	: std_logic;					--pamet stavu cCGAHOR(2) v minulem CLK
signal pWRREQ	: std_logic;					--pamet pozadavku zapisu

--vnitrni signaly pro VGA-----------------------------------------------------
signal cVGAHOR	: std_logic_vector(10 downto 0);		--horizontalni citac
signal cVGAVER	: std_logic_vector(9 downto 0);			--vertikalni citac
signal rVIDEN	: std_logic;					--povoleni zobrazovani
signal vENVER	: std_logic;					--povoleni zobrazovani od vertikalniho citace
signal rVGASEL	: std_logic;					--vyber stranky pameti pro cteni
signal rDIN	: std_logic_vector(15 downto 0);		--register prijatych dat z RAM

begin

--generovani zapisove adresy a prijem barvonosne informace od MZ800-----------
process (CGA_CLK) begin
    if CGA_CLK'event and CGA_CLK='1' then
	rCGAHS <= CGA_HS;					--pamet stavu horizontalni synchronizace
	rCGAVS <= CGA_VS;					--pamet stavu vertikalni synchronizace
	if rCGAHS='0' and CGA_HS='1' then			--test na konec CGA hsync
	    cCGAHOR <= "1101001011";				--synchronizace cCGAHOR citace
	    cCGAVER <= cCGAVER + 1;				--presun na dalsi radek
	else
	    cCGAHOR <= cCGAHOR + 1;				--presun na dalsi pixel
	end if;
	if rCGAVS='0' and CGA_VS='1' then			--test na konec CGA vsync
	   cCGAVER <= "111101100";				--synchronizace cCGAVER citace
	   rCGASEL <= not rCGASEL;				--preklopeni do druhe poloviny pameti
	end if;
	if cCGAHOR(1 downto 0)="00" then
	    r3RGBI(3 downto 0) <= CGA_RGBI;			--ulozi barvy prvniho pixelu
	elsif cCGAHOR(1 downto 0)="01" then
	    r3RGBI(7 downto 4) <= CGA_RGBI;			--ulozi barvy druheho pixelu
	elsif cCGAHOR(1 downto 0)="10" then
	    r3RGBI(11 downto 8) <= CGA_RGBI;			--ulozi barvy tretiho pixelu
	else
	    r4RGBI <= CGA_RGBI & r3RGBI;			--ulozi barvy ctyr pixelu (zapisovy registr pro pamet)
	    rCGAADR <= cCGAHOR(9 downto 2);			--zapamatuje si stav cCGAHOR kvuli stabilni adresaci pameti pri zapisu
								--POZOR bez tohoto se v obraze tvori DropOuts
	end if;
    end if;
end process;

process (VGA_CLK) begin
    if VGA_CLK'event and VGA_CLK='0' then

--generovani signalu WR pro pamet RAM-----------------------------------------
	iRAM_WR <= '1';						--zapis je defaultne neaktivni
	pWRREQ <= '0';						--pamet pozadavku zapisu je defaultne neaktivni
	if cCGAHOR(2)/=rWRCOM or pWRREQ='1' then		--detekce pozadavku o zapis
	    if cVGAHOR(1 downto 0)="10" then			--pokud bude nasledovat cteni z RAM
		pWRREQ <= '1';					--zapis nemuze byt vykonan nastavi se pamet pozadavku zapisu
	    else
		iRAM_WR <= '0';					--nastav signal /WR
	    end if;
	end if;
	rWRCOM <= cCGAHOR(2);					--pamet stavu cCGAHOR(2)

--citani, nulovani horizontalniho a vertikalniho vga citace-------------------
	if cVGAHOR=1055 then					--pokud jsme na poslednim bode radku
	    cVGAHOR <= (others=>'0');				--nulovani horizontalniho citace
	    if cVGAVER=627 then					--pokud jsme na poslednim radku
		cVGAVER <= (others=>'0');			--nulovani vertikalniho citace
		rVGASEL <= not rCGASEL;				--preklopeni do pameti do ktere se nezapisuje
	    else
		cVGAVER <= cVGAVER + '1';			--vertikalni citac +1
	    end if;
	else
	    cVGAHOR <= cVGAHOR + '1';				--horizontalni citac +1
	end if;
--vertikalni synchronizace vga------------------------------------------------
	if cVGAVER=0 then
	    vENVER <= '1';					--povoleni zobrazeni
	elsif cVGAVER=572 then
	    vENVER <= '0';					--zakazani zobrazeni
	elsif cVGAVER=587 then
	    VGA_VS <= '1';					--zacatek vertikalni synchronizace
	elsif cVGAVER=591 then
	    VGA_VS <= '0';					--konec vertikalni synchronizace
	end if;
--horizontalni synchronizace vga----------------------------------------------
	if cVGAHOR=1 and vENVER='1' then
	    rVIDEN <= '1';					--povoleni zobrazovani
	elsif cVGAHOR=801 then
	    rVIDEN <= '0';					--zakazani zobrazovani
	elsif cVGAHOR=841 then
	    VGA_HS <= '1';					--zacatek horizontalni synchronizace
	elsif cVGAHOR=969 then
	    VGA_HS <= '0';					--konec horizontalni synchronizace
	end if;
--zapis do registru prijatych dat z RAM---------------------------------------
	if cVGAHOR(1 downto 0)="11" then
	    rDIN <= RAM_DT;
	end if;
    end if;
end process;

--generovani barvonosnych signalu---------------------------------------------
--POZOR pokud jsem se snazil nasledujici generovat 
--posuvnym registrem - v obraze se tvori DropOuts
VGA_R <=	'0' when rVIDEN='0' else			--vystup vga RED
		rDIN(0) when cVGAHOR(1 downto 0)="00" else	--prvni bod
		rDIN(4) when cVGAHOR(1 downto 0)="01" else	--druhy bod
		rDIN(8) when cVGAHOR(1 downto 0)="10" else	--treti bod
		rDIN(12);					--ctvrty bod
VGA_G <=	'0' when rVIDEN='0' else			--vystup vga GREEN
		rDIN(1) when cVGAHOR(1 downto 0)="00" else	--prvni bod
		rDIN(5) when cVGAHOR(1 downto 0)="01" else	--druhy bod
		rDIN(9) when cVGAHOR(1 downto 0)="10" else	--treti bod
		rDIN(13);					--ctvrty bod
VGA_B <=	'0' when rVIDEN='0' else			--vystup vga BLUE
		rDIN(2) when cVGAHOR(1 downto 0)="00" else	--prvni bod
		rDIN(6) when cVGAHOR(1 downto 0)="01" else	--druhy bod
		rDIN(10) when cVGAHOR(1 downto 0)="10" else	--treti bod
		rDIN(14);					--ctvrty bod
VGA_I <=	'0' when rVIDEN='0' else			--vystup vga intenzita
		rDIN(3) when cVGAHOR(1 downto 0)="00" else	--prvni bod
		rDIN(7) when cVGAHOR(1 downto 0)="01" else	--druhy bod
		rDIN(11) when cVGAHOR(1 downto 0)="10" else	--treti bod
		rDIN(15);					--ctvrty bod

--napojeni pameti RAM---------------------------------------------------------
RAM_AD <= rVGASEL & cVGAVER(9 downto 1) & cVGAHOR(9 downto 2) when cVGAHOR(1 downto 0)="11" else
	  rCGASEL & cCGAVER(8 downto 0) & rCGAADR;
RAM_DT <= r4RGBI when iRAM_WR='0' else "ZZZZZZZZZZZZZZZZ";
RAM_WR <= iRAM_WR;

end rtl;
